# -*- coding: utf-8 -*-
#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

from io import open
from aceql._private.datetime_util import *
from aceql._private.file_util import *
from aceql.sql_null_type import *


class CursorUtil(object):
    """Utilities for Cursor class. """

    JAVA_MAX_INT_VALUE = 2147483647

    def __init__(self):
        self.blob_ids = []
        self.blob_streams = []
        self.blob_lengths = []

    def get_http_parameters_dict(self, params):
        """ Build the http parameters dictionary to pass to remote server. """
        parms_dict = {}
        param_index = 0

        # Blob lists

        if params is None:
            return parms_dict

        for x in params:
            param_index += 1

            # For Py2 always convert "unicode" aka str with special chars to str
            x = CursorUtil.get_utf8_value(x)

            param_type = CursorUtil.get_sql_type(x)
            parms_dict["param_type_" + str(param_index)] = param_type

            # NULL values are defined in a (None, SqlNullType.TYPE) typle
            # BLOB values are defined in a (fd,) or (fd, length) tuple where
            # fd = open(filename)
            if CursorUtil.get_class_name(x) == "tuple":
                if param_type == "BLOB":
                    blob_id = FileUtil.get_unique_id() + ".blob"
                    parms_dict["param_value_" + str(param_index)] = blob_id

                    self.blob_ids.append(blob_id)
                    self.blob_streams.append(x[0])

                    if len(x) == 2:
                        self.blob_lengths.append(x[1])
                    else:
                        self.blob_lengths.append(0)
                else:
                    parms_dict["param_value_" + str(param_index)] = "NULL"

            elif CursorUtil.get_class_name(x) == "datetime.datetime":
                parms_dict["param_value_" + str(param_index)] = DateTimeUtil.get_timestamp_from_date(x)
            elif CursorUtil.get_class_name(x) == "datetime.date":
                parms_dict["param_value_" + str(param_index)] = DateTimeUtil.get_timestamp_from_date(x)
            elif CursorUtil.get_class_name(x) == "datetime.time":
                the_datetime = datetime.now()
                the_datetime.hour = x.hour
                the_datetime.minute = x.minute
                the_datetime.second = x.second
                parms_dict["param_value_" + str(param_index)] = DateTimeUtil.get_timestamp_from_date(the_datetime)
            else:
                parms_dict["param_value_" + str(param_index)] = str(x)

        return parms_dict

    def get_utf8_value(x):
        """ For python 2: string values with special chars must be UTF-8 encoded """
        if FileUtil.is_python_2() and CursorUtil.get_class_name(x) == "unicode":
            x = x.encode('utf-8')
        return x

    get_utf8_value = staticmethod(get_utf8_value)

    # None / <class 'NoneType'>
    # 1 / <class 'int'>
    # 12.53 / <class 'float'>
    # True / <class 'bool'>
    # text / <class 'str'>
    # 2005-07-14 00:00:00 / <class 'datetime.datetime'>
    # 2005-07-14 / <class 'datetime.date'>
    # 12:30:00 / <class 'datetime.time'>

    def get_sql_type(x):
        """get the SQL type of the passed param value. """

        if x is None:
            raise TypeError("Parameter value is None!")

        if CursorUtil.get_class_name(x) == "tuple":
            if x[0] is None:
                # HACK for BOOL that must be converted to BIT
                # NO! Do not know why. Comment it:
                #if x[1] == SqlNullType.BLOB:
                #   x[1] = SqlNullType.BIT
                sql_type = "TYPE_NULL" + str(x[1])
            elif CursorUtil.get_class_name(x[0]) == "_io.BufferedReader":
                sql_type = "BLOB"
            elif CursorUtil.get_class_name(x[0]) == "file":
                sql_type = "BLOB"
            else:
                raise TypeError("Invalid tuple parameter. Not a NULL Type nor a BLOB: " + str(x))
        # Because P2 int and long are different, and p3 int and long are awlays int, special treatment:
        elif CursorUtil.get_class_name(x) == "int" or CursorUtil.get_class_name(x) == "long":
            if x < CursorUtil.JAVA_MAX_INT_VALUE * -1 or x >  CursorUtil.JAVA_MAX_INT_VALUE:
                sql_type = "BIGINT"
            else:
                sql_type = "INTEGER"
        elif CursorUtil.get_class_name(x) == "bool":
            sql_type = "BIT"
        elif CursorUtil.get_class_name(x) == "float":
            sql_type = "REAL"
        elif CursorUtil.get_class_name(x) == "str":
            sql_type = "VARCHAR"
        # NO! unicode is translate to str previously
        # elif CursorUtil.get_class_name(x) == "unicode":
        #    sql_type = "VARCHAR"
        elif CursorUtil.get_class_name(x) == "datetime.datetime":
            sql_type = "TIMESTAMP"
        elif CursorUtil.get_class_name(x) == "datetime.date":
            sql_type = "DATE"
        elif CursorUtil.get_class_name(x) == "datetime.time":
            sql_type = "TIME"
        else:
            print("CursorUtil.get_class_name(x): " + CursorUtil.get_class_name(x))
            raise TypeError("Type is not supported for value: " + str(x))

        return sql_type

    get_sql_type = staticmethod(get_sql_type)

    def get_class_name(x):
        """ Parse <class 'class_name'> to get only class_name. """

        s = str(type(x))
        s = s.replace("<class '", "")
        s = s.replace("<type '", "")  # Python2 syntax
        s = s[0:len(s) - 2]
        return s

    get_class_name = staticmethod(get_class_name)



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

import json


class ResultAnalyzer(object):
    """ <summary>
     Class <see cref="ResultAnalyzer"/>. Used to analyze a JSON response from the AceQL server.
     </summary>
    """

    # <summary>
    # The json result
    # </summary>
    # <summary>
    # We try to find status.  If error parsing, invalidJsonStream = true
    # </summary>
    # * Exception when parsing the JSON stream.  Future usage
    def __init__(self, son_result, http_status_code):
        """ <summary>
         Initializes a new instance of the <see cref="ResultAnalyzer"/> class.
         </summary>
         <param name="son_result">The json result.</param>
         <param name="http_status_code">The http status code.</param>
         <exception cref="System.ArgumentNullException">son_result is null!</exception>
        """
        self._invalid_json_stream = False
        self._parse_exception = None
        self._http_status_code = http_status_code
        self._json_result = son_result

    def is_status_ok(self):
        """ <summary>
         Determines whether the SQL command correctly executed on server side.
         </summary>
         <returns><c>true</c> if [is status ok]; otherwise, <c>false</c>.</returns>
        """

        if self._json_result is None or len(self._json_result) == 0:
            return False
        try:
            j = json.loads(self._json_result)
            status = j["status"]
            if status == "OK":
                return True
            else:
                return False
        except Exception as e:
            self._parse_exception = e
            self._invalid_json_stream = True
            return False

    def is_invalid_json_stream(self):
        """ <summary>
         Says if the JSON Stream is invalid.
         </summary>
         <returns>true if JSN stream is invalid</returns>
        """
        if self._json_result is None or len(self._json_result) == 0:
            return True
        if self._invalid_json_stream:
            return True
        return False

    def get_result(self, name):
        """ <summary>
         Gets the result for a a key name
         </summary>
         <param name="name">The name.</param>
         <returns>System.String.</returns>
        """
        return self.get_value(name)

    def get_result(self):
        """ <summary>
         Gets the result for the key name "result"
         </summary>
         <returns></returns>
        """
        return self.get_value("result")

    def get_value(self, name):
        """ <summary>
         Gets the value.
         </summary>
         <param name="name">The name.</param>
         <returns>System.String.</returns>
         <exception cref="System.ArgumentNullException">name is null!</exception>
         <exception cref="System.Exception">Illegal name: " + name</exception>
        """
        if name is None:
            raise TypeError("name is null!")
        if self.is_invalid_json_stream():
            return None

        try:
            j = json.loads(self._json_result)

            if name == "session_id":
                value = j[name]
            elif name == "connection_id":
                value = j[name]
            elif name == "length":
                value = j[name]
            elif name == "result":
                value = j[name]
            elif name == "row_count":
                value = j[name]
            else:
                raise Exception("Illegal name: " + name)
            return value
        except Exception as e:
            self._parse_exception = e
            self._invalid_json_stream = True
            return None

    def get_error_type(self):
        """ <summary>
         Gets the error_type.
         </summary>
         <returns>System.Int32.</returns>
        """
        if self.is_invalid_json_stream():
            return 0

        try:
            j = json.loads(self._json_result)
            error_type = j["error_type"]
            return error_type
        except Exception as e:
            self._parse_exception = e
            self._invalid_json_stream = True
            return 0

    def get_error_message(self):
        """ <summary>
         Gets the error_message.
         </summary>
         <returns>System.String.</returns>
        """
        if self.is_invalid_json_stream():
            the_error_message = "Unknown error."
            if self._http_status_code != 200:
                the_error_message = "HTTP FAILURE " + str(self._http_status_code);
            return the_error_message
        try:
            j = json.loads(self._json_result)
            error_message = j["error_message"]
            return error_message
        except Exception as e:
            self._parse_exception = e
            self._invalid_json_stream = True
            return None

    def get_stack_trace(self):
        """ <summary>
         Gets the remote stack_trace.
         </summary>
         <returns>String.</returns>
        """
        if self.is_invalid_json_stream():
            return None
        try:
            j = json.loads(self._json_result)
            stack_trace = j["stack_trace"]
            return stack_trace
        except Exception as e:
            self._parse_exception = e
            self._invalid_json_stream = True
            return None

    def get_int_value(self, name):
        """ <summary>
         Gets the int value.
         </summary>
         <param name="name">The name.</param>
         <returns>System.Int32.</returns>
        """
        ins_str = self.get_value(name)
        if ins_str is None:
            return -1
        return int(ins_str)



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

from datetime import datetime, time
import time
import pytz


class DateTimeUtil(object):
    """Date & datetime utilities

    Includes conversion from and to Unix Timestamp as date/time object
    are transferred as a long Unix Epoch
    """

    # def get_timestamp_from_datetime(the_date_time):
    #
    # 	# "Returns an Epoch for a datetime."
    # 	# ts = the_date_time.timestamp()
    # 	# tsStr = str(ts * 1000)
    # 	# dotIndex = tsStr.index(".")
    # 	# tsStr = tsStr[0: dotIndex]
    #    #
    # 	# print ("tsStr: " + tsStr)
    # 	# return tsStr

    # get_timestamp_from_datetime = staticmethod(get_timestamp_from_datetime)

    def to_timestamp(a_date):
        if a_date.tzinfo:
            epoch = datetime(1970, 1, 1, tzinfo=pytz.UTC)
            diff = a_date.astimezone(pytz.UTC) - epoch
        else:
            epoch = datetime(1970, 1, 1)
            diff = a_date - epoch
        return int(diff.total_seconds())

    to_timestamp = staticmethod(to_timestamp)

    def get_timestamp_from_date(the_date):
        """"Returns an Epoch for date."""
        unix_time = time.mktime(the_date.timetuple())

        unix_time_str = str(unix_time)
        unix_time_str = unix_time_str[0:len(unix_time_str) - 2]
        return unix_time_str + "000"

    get_timestamp_from_date = staticmethod(get_timestamp_from_date)

    def get_datetime_from_timestamp(ts_str):
        """Returns a datetime from Unix Epoch."""

        ts = float(ts_str)
        ts_int = ts / 1000  # we want in seconds
        return datetime.fromtimestamp(ts_int)

    get_datetime_from_timestamp = staticmethod(get_datetime_from_timestamp)

    def get_date_from_timestamp(ts_str):
        """Returns a date from Unix Epoch."""

        ts = float(ts_str)
        ts_int = ts / 1000  # we want in seconds
        the_datetime = datetime.fromtimestamp(ts_int)
        return the_datetime.date()

    get_date_from_timestamp = staticmethod(get_date_from_timestamp)

    def get_time_from_timestamp(ts_str):
        """Returns a date from Unix Epoch."""

        ts = float(ts_str)
        ts_int = ts / 1000  # we want in seconds
        the_datetime = datetime.fromtimestamp(ts_int)
        return the_datetime.time()

        get_time_from_timestamp = staticmethod(get_time_from_timestamp)



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

from aceql.cursor import *


class Connection(object):
    """Allows to create a database connection to a remote server."""

    def __init__(self, server_url, database, username, password, session_id=None, proxies=None, auth=None):
        """
        Creates a database connection to the remote AceQL HTTP server.

        Parameters
        ----------
        server_url : str
            The URL of the AceQL server. Example: https://www.acme.com:9443/aceql.
        database : str
            The remote database name.
        username : str
            The authentication username.
        password : str
            the authentication password.
        session_id : str
            the session_id generated by AceQL login method on server. To be used for authentication without password.
        proxies : str
            The proxy to use, can  be an authenticated proxy.
        auth:   : ProxyAuth
            aceql.ProxyAuth instance with proxy (username, password)

        Returns
        -------
        Connection
            A connection to the remote database.
            :param session_id:

        """

        if server_url is None:
            raise TypeError("server_url is null!")
        if database is None:
            raise TypeError("database is null!")
        if username is None:
            raise TypeError("username is null!")
        if password is None and session_id is None:
            raise TypeError("password and session_id are both null!")

        self.__aceQLHttpApi = AceQLHttpApi(server_url, database, username, password, session_id, proxies=proxies, auth=auth)

    @property
    def _get_aceql_http_api(self):
        return self.__aceQLHttpApi

    def cursor(self):
        """Instantiates and returns a cursor."""
        cursor = Cursor(self, self.__aceQLHttpApi)
        return cursor

    def is_stateless():
        """Says if session is stateless."""
        return AceQLHttpApi.is_stateless()

    is_stateless = staticmethod(is_stateless)

    def set_stateless(stateless):
        """Sets the session mode. if true, the session will be stateless, else stateful."""
        if stateless is None:
            raise TypeError("stateless is null!")
        if str(stateless) == "True":
            AceQLHttpApi.set_stateless(True)
        else:
            AceQLHttpApi.set_stateless(False)

    set_stateless = staticmethod(set_stateless)

    def set_timeout(timeout):
        """ Sets the HTTP connection timeout in seconds.
        0 means not timeout is used (default value)."""

        if timeout is None:
            raise TypeError("timeout is null!")

        AceQLHttpApi.set_timeout(timeout)

    set_timeout = staticmethod(set_timeout)

    def set_progress_indicator(self, progress_indicator):
        """ Allows to set a progress indicator."""
        self.__aceQLHttpApi.set_progress_indicator(progress_indicator)

    def get_progress_indicator(self):
        """Returns the progress indicator in use."""
        return self.__aceQLHttpApi.get_progress_indicator()

    def set_auto_commit(self, auto_commit):
        """Sets this connection's auto-commit mode to the given state."""
        self.__aceQLHttpApi.set_auto_commit(auto_commit)

    def get_auto_commit(self):
        """Retrieves the current auto-commit mode."""
        return self.__aceQLHttpApi.get_auto_commit()

    def commit(self):
        """Commit current transaction."""
        self.__aceQLHttpApi.commit()

    def rollback(self):
        """Rollback current transaction."""
        self.__aceQLHttpApi.rollback()

    def _trace(self):
        """Print empty line on trace."""
        self.__aceQLHttpApi.trace()

    def trace(self, s):
        """Prints the string on trace."""
        self.__aceQLHttpApi.trace(s)

    def _is_trace_on():
        """Says if trace is on."""
        return AceQLHttpApi.is_trace_on()

    _is_trace_on = staticmethod(_is_trace_on)

    def _set_trace_on(trace_on):
        """Sets the trace on/off."""
        AceQLHttpApi.set_trace_on(trace_on)

    _set_trace_on = staticmethod(_set_trace_on)

    def is_gzip_result(self):
        """Says if the query result is returned compressed with the GZIP file format."""
        return self.__aceQLHttpApi.is_gzip_result()

    def set_gzip_result(self, gzip_result):
        """Define if result sets are compressed before download. Defaults to true."""
        self.__aceQLHttpApi.set_gzip_result(gzip_result)

    def get_server_version(self):
        """Gets the server version of AceQL HTTP."""
        return self.__aceQLHttpApi.get_server_version()

    def get_client_version(self):
        """Gets the SDK version."""
        return self.__aceQLHttpApi.get_client_version()

    def close(self):
        """Closes the connection to the remote database but keeps the HTTP session."""
        self.__aceQLHttpApi.close()

    def logout(self):
        """Closes all the connection to the remote database and closes the HTTP session."""
        self.__aceQLHttpApi.logout()

    def get_transaction_isolation(self):
        """Returns the current transaction isolation level."

           Will be one of the following constants:
                transaction_read_uncommitted,
                transaction_read_committed,
                transaction_repeatable_read,
                transaction_serializable, or
                transaction_none.
        """
        return self.__aceQLHttpApi.get_transaction_isolation()

    def set_transaction_isolation(self, level):
        """Sets the transaction isolation level."""
        self.__aceQLHttpApi.set_transaction_isolation(level)

    def get_holdability(self):
        """Returns the holdability.
         One of hold_cursors_over_commit or close_cursors_at_commit.
        """
        return self.__aceQLHttpApi.get_holdability()

    def set_holdability(self, holdability):
        """Sets the holdability."""
        self.__aceQLHttpApi.set_holdability(holdability)

    def is_read_only(self):
        """Says if Connection is read-only."""
        return self.__aceQLHttpApi.is_read_only()

    def set_read_only(self, read_only):
        """Allows to put Connection read-only mode."""
        self.__aceQLHttpApi.set_read_only(read_only)



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

from aceql._private.aceql_http_api import *
from aceql._private.file_util import *


class AceQLHttpApiTest(object):
    """description of class"""

    def doIt(self):
        AceQLHttpApi.set_stateless(True)
        is_stateless = AceQLHttpApi.is_stateless()
        print("isStateless: " + str(is_stateless))

        AceQLHttpApi.set_stateless(False)
        is_stateless = AceQLHttpApi.is_stateless()
        print("isStateless: " + str(is_stateless))

        aceql_http_api = AceQLHttpApi("http://localhost:9090/aceql", "sampledb", "user1", "password1")
        print("connect done!")

        print("client version: " + aceql_http_api.get_client_version())
        print("server version: " + aceql_http_api.get_server_version())

        auto_commit = aceql_http_api.get_auto_commit()
        print("auto_commit: " + str(auto_commit))

        aceql_http_api.set_auto_commit(False)
        auto_commit = aceql_http_api.get_auto_commit()
        print("auto_commit: " + str(auto_commit))

        aceql_http_api.commit()
        aceql_http_api.rollback()
        aceql_http_api.set_auto_commit(True)

        auto_commit = aceql_http_api.get_auto_commit()
        print("auto_commit: " + str(auto_commit))

        holdability = aceql_http_api.get_holdability()
        print("holdability: " + holdability)

        transaction_isolation = aceql_http_api.get_transaction_isolation()
        print("transaction_isolation: " + transaction_isolation)

        read_only = aceql_http_api.is_read_only()
        print("readOnly: " + str(read_only))

        print()
        sql = "update customer set fname = ? where customer_id = ?"
        is_prepared_statement = True

        statement_parameters = {}
        statement_parameters["param_type_1"] = "VARCHAR"
        statement_parameters["param_value_1"] = "Nicolas"
        statement_parameters["param_type_2"] = "INTEGER"
        statement_parameters["param_value_2"] = "1"

        result = aceql_http_api.execute_update(sql, is_prepared_statement, statement_parameters)
        print("result: " + str(result))

        status_code = aceql_http_api.get_http_status_code()
        print("statusCode: " + str(status_code))

        status_message = aceql_http_api.get_http_status_message()
        print("status_message: " + str(status_message))
        print()

        print("UUID: " + FileUtil.get_unique_id())
        print("Home: " + FileUtil.get_user_home_dot_kawansoft_dir())
        print("Tmp : " + FileUtil.get_kawansoft_temp_dir())
        print()

        statement_parameters2 = {}
        statement_parameters2["param_type_1"] = "INTEGER"
        statement_parameters2["param_value_1"] = "0"
        sql = "select * from customer where customer_id >= ? order by customer_id"
        aceql_http_api.set_pretty_printing(True)

        result = aceql_http_api.execute_query(sql, is_prepared_statement, statement_parameters2)
        print("result: " + str(result))
        print()



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

"""
This module implements some constructors and singletons as required by the
DB API v2.0 (PEP-249).
"""

import time
import datetime

# Python Db API v2
apilevel = '2.0'
threadsafety = 1
paramstyle = 'qmark'

Date = datetime.date
Time = datetime.time
Timestamp = datetime.datetime

def DateFromTicks(ticks):
    return Date(*time.localtime(ticks)[:3])


def TimeFromTicks(ticks):
    return Time(*time.localtime(ticks)[3:6])


def TimestampFromTicks(ticks):
    return Timestamp(*time.localtime(ticks)[:6])


# We define the blob a string, see user manual
Binary = "BLOB"

# There are no Types Objects required by AceQL HTTP



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from aceql._private.user_login_store import *

user_login_store= UserLoginStore("http://localhost:9090/aceql", "username", "sampledb")
user_login_store.set_session_id("session_id_value")

if user_login_store.is_already_logged():
    print("User is already logged!")
    print("sesssion_id: " + user_login_store.get_session_id())
else:
    print("User is not logged.")

user_login_store.remove()

if user_login_store.is_already_logged():
    print("User is already logged!")
    print("sesssion_id: " + user_login_store.get_session_id())
else:
    print("User is not logged.")






from dataclasses import dataclass, field
from typing import List, Optional, ClassVar
from typing import ClassVar, Type

import marshmallow_dataclass
import marshmallow.validate
from marshmallow import Schema
from typing_extensions import Type


@dataclass
class Tables:
    # field metadata is used to instantiate the marshmallow field
    status: str
    tableNames: List[str]
    toto: bool
    Schema: ClassVar[Type[Schema]] = Schema

    def __str__(self):
        """ The string representation."""
        return str(self.status) + ", " + str(self.tableNames)


@dataclass
class Building():
    # field metadata is used to instantiate the marshmallow field
    height: float
    name: str

    class Meta:
        ordered = True

    def __str__(self):
        """ The string representation."""
        return str(self.height) + ", " + str(self.name)

@dataclass
class City():
    name: Optional[str]
    buildings: List[Building]
    is_high: bool

    class Meta:
        ordered = True

    def get_name(self):
        return self.name

    def __str__(self):
        """ The string representation."""
        return str(self.name) + ", " + str(self.buildings) + "," + str(self.is_high)

f = open("C:\\test\\json_test.txt","r")
jsonString = f.read()
f.close()
print()
print(jsonString)

CitySchema = marshmallow_dataclass.class_schema(City )

# city = CitySchema().load(
#     {"name": "Paris", "buildings": [{"name": "Eiffel Tower", "height": 324}], "is_high": False}
# )
city : City = CitySchema().loads(jsonString)

# => City(name='Paris', buildings=[Building(height=324.0, name='Eiffel Tower')])
print(city.get_name())

# city_dict = CitySchema().dump(city)
# # => {'name': 'Paris', 'buildings': [{'name': 'Eiffel Tower', 'height': 324.0}]}
# print(city_dict)

# TablesSchema = marshmallow_dataclass.class_schema(Tables)
#
# f = open("C:\\test\\table_names_json.txt","r")
# jsonString = f.read()
# f.close()
# print()
# print(jsonString)
#
# tables =  TablesSchema().load(
#     {"status": "OK", "tableNames": ["banned_usernames", "customer"] }
# )
# print(tables)
#
# #string = "{\"status\": \"OK\", \"tableNames\": [\"banned_usernames\", \"customer\"] }"
# tables =  TablesSchema().loads(jsonString)
# print(tables)


#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

import collections
import json
import os
from io import open

from aceql._private import cursor_util
from aceql._private.column_types_builder import *
from aceql._private.aceal_debug import *
from aceql._private.cursor_util import *


def is_row_one(s):
    return s == "\"row_1\":[" or s == "\"row_1\": ["


class RowParser(object):
    """Allows to parse rows in retrieved JSON result set and return each row content dictionaries"""

    def __init__(self, filename, row_count):
        self.__filename = filename
        self.__last_row = 1
        self.__row_count = row_count
        self.__rows_parsed = 0

        self.__column_names_per_index = {}
        self.__values_per_col_index = {}
        self.__types_per_col_index = {}

        if filename is None:
            raise TypeError("filename is null!")

        if not os.path.isfile(filename):
            raise IOError("filename does not exist: " + str(filename))

        # Builds the column types
        column_types_builder = ColumnTypesBuilder(filename)
        self.__types_per_col_index = column_types_builder.get_types_per_col_index()

        self.__fd = open(filename, mode="r", encoding="utf-8")

        # Go to row_1
        while True:
            s = self.__fd.readline()
            if s == '':
                break
            s = s.strip()
            if is_row_one(s):
                break

    def build_next_row(self):
        """Build the dictionaries of values per column name & values per column index for the next row"""

        # No parse of course if no rows in file...
        if self.__row_count == 0:
            return False

        # Stop parsing if end of file reached
        if self.__rows_parsed >= self.__row_count:
            self.__column_names_per_index = {}
            self.__values_per_col_index = {}
            return False

        s = ""
        while True:
            line = self.__fd.readline()
            if line == '':
                break
            line = line.strip()
            if self.is_last_row(line):
                self.__last_row += 1
                break

            # Case last row
            if self.__last_row == self.__row_count:
                if line == "]":
                    self.__last_row += 1
                    break

            # doe not include closing array bracket
            if not line.startswith("]"):
                s += line

        if s == '':
            return

        # AceQLDebug.print("s:" + s)
        AceQLDebug.debug("")
        AceQLDebug.debug("s               : " + s)
        AceQLDebug.debug("self.__last_row : " + str(self.__last_row))

        s = s.replace("{", "")
        s = s.replace("}", "")
        s = "{" + s + "}"
        resp = json.loads(s, object_pairs_hook=collections.OrderedDict)

        index = 0
        for key, value in resp.items():
            # AceQLDebug.debug("key/value: " + str(key) + " " + str(value))
            self.__column_names_per_index[index] = key

            x = CursorUtil.get_utf8_value(value)
            if str(x) == 'NULL':
                self.__values_per_col_index[index] = None
            else:
                self.__values_per_col_index[index] = value

            index += 1
        # AceQLDebug.debug("key: %s , value: %s" % (key, self.__values_per_col_name [key]))

        self.__rows_parsed += 1
        return True

    def is_last_row(self, line):
        # if line == ("\"row_" + str(self.__last_row + 1) + "\":["):
        return line == ("\"row_" + str(self.__last_row + 1) + "\":[") or line == (
                    "\"row_" + str(self.__last_row + 1) + "\": [")

    def column_names_per_index(self):
        """Returns the dictionary of column names per column index, starting at 0 """
        return self.__column_names_per_index

    def get_values_per_col_index(self):
        """Returns the dictionary of values per column index, starting at 0 """
        return self.__values_per_col_index

    def get_types_per_col_index(self):
        """Returns the dictionary of type per column index, starting at 0 """
        return self.__types_per_col_index

    def get_row_cout(self):
        return self.__row_count

    def close(self):
        """Mandatory close at end of use"""
        self.__fd.close()



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

import os
from io import open

import ijson

from aceql._private.aceal_debug import *


class RowCounter(object):
    """Allows to count rows in retrieved JSON result set"""

    def __init__(self, filename):
        self.__filename = filename

        if filename is None:
            raise TypeError("filename is null!")

        if not os.path.isfile(filename):
            raise IOError("filename does not exist: " + str(filename))

    def count(self):
        """Returns the number of rows in JSON file (key "row_count") """
        rows = 0
        with open(self.__filename, 'rb') as input_file:
            objects = ijson.items(input_file, 'row_count')
            elements = (o for o in objects)
            for element in elements:
                rows = int(element)
                break
            return rows




#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

import requests
from requests_toolbelt.multipart import encoder

from aceql.error import *
from aceql._private.result_analyzer import *
from aceql._private.result_set_info import *
from aceql._private.row_counter import *
from aceql._private.stream_result_analyzer import *
from aceql._private.version_values import *


class UserLoginStore(object):
    """ Stores the session Id per server_url/username/database triplet in order to
        get new AceQL Connection with /get_connection without new login action."""

    __logged_users = {}

    def __init__(self, server_url, username, database):
        """Constructor"""

        if server_url is None:
            raise TypeError("serverUrl is null!")
        if username is None:
            raise TypeError("username is null!")
        if database is None:
            raise TypeError("database is null!")

        self.__server_url = server_url
        self.__username = username
        self.__database = database

    def build_key(self):
        """Builds the Dcit key for the (:, username, database) triplet key."""
        return self.__server_url + "/" + self.__username + "/" + self.__database;

    def is_already_logged(self):
        """Says if user is already logged, aka key exists in Dict"""
        key = self.build_key()
        data = UserLoginStore.__logged_users.get(key)
        if data is None:
            return False
        else:
            return True

    def get_session_id(self):
        """Returns the session Id of logged user with (server_url, username, database) triplet."""
        key = self.build_key()
        session_id = UserLoginStore.__logged_users[key]
        return session_id

    def set_session_id(self, session_id):
        """Stores the session Id of a logged user with (server_url, username, database) triplet."""
        key = self.build_key()
        UserLoginStore.__logged_users[key] = session_id

    def remove(self):
        """Removes (server_url, username, database) triplet. This is to be called at /logout API.
        """
        key = self.build_key()
        del UserLoginStore.__logged_users[key]



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from requests_toolbelt.auth.guess import GuessProxyAuth


class ProxyAuth(GuessProxyAuth):
    """ Allows to set authenticated proxy username and password.

        Class inherits requests_toolbelt GuessProxyAuth and GuessAuth.
        This allows to pass and use directly parent instances if necessary.
    """

    def __init__(self,
                 proxy_username=None, proxy_password=None):
        super(ProxyAuth, self).__init__('user', 'passwd')
        self.proxy_username  = proxy_username
        self.proxy_password = proxy_password






#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

from aceql._private.row_parser import *
from aceql._private.aceql_http_api import *


class Cursor(object):
    """Cursor class."""

    def __init__(self, connection, aceql_http_api):
        self.__connection = connection
        self.__aceql_http_api = aceql_http_api
        self.__is_closed = False

        self.__rowcount = -1
        self.__description = []

        self.__filelist = []
        self.arraysize = 1

        self.__row_parser = None

    @property
    def description(self):
        """Describes the name and SLQ type of each column.

        (5 other elements are not set in this version)"""

        self.__raise_error_if_closed()
        return self.__description

    @property
    def rowcount(self):
        """This read-only attribute specifies the number of rows that the last .execute*()
        produced (for DQL statements like SELECT)
        or affected (for DML statements like UPDATE or INSERT)"""
        self.__raise_error_if_closed()
        return self.__rowcount

    def execute(self, sql, params=()):
        """Executes the given operation

        Executes the given operation substituting any markers with
        the given parameters.

        For example, getting all rows where id is 5:
          cursor.execute("SELECT * FROM t1 WHERE id = ?", (5,))
        """
        self.__raise_error_if_closed()

        if sql is None:
            raise TypeError("sql is null!")

        sql = sql.strip()

        if sql.lower().startswith("select"):
            return self.__execute_query(sql, params)
        else:
            return self.__execute_update(sql, params)

    def executemany(self):
        """Execute the given operation multiple times.

        Not implemented in this version.
        """
        self.__raise_error_if_closed()
        raise NotImplementedError("executemany is not implemented in this AceQL SDK version.")

    def __execute_update(self, sql, params=()):
        """Executes and update operation on remote database"""

        blob_streams = []

        try:
            the_cursor_util = CursorUtil()
            parms_dict = the_cursor_util.get_http_parameters_dict(params)

            blob_ids = the_cursor_util.blob_ids
            blob_streams = the_cursor_util.blob_streams
            blob_lengths = the_cursor_util.blob_lengths

            cpt = 0
            for blob_id in blob_ids:
                self.__aceql_http_api.blob_upload(blob_id, blob_streams[cpt], blob_lengths[cpt])
                cpt += 1

            AceQLDebug.debug("parms_dict: " + str(parms_dict))

            is_prepared_statement = False
            if len(parms_dict) > 0:
                is_prepared_statement = True

            rows = self.__aceql_http_api.execute_update(sql, is_prepared_statement, parms_dict)
            self.__rowcount = rows
            return rows
        finally:
            for blob_stream in blob_streams:
                blob_stream.close()

    def __execute_query(self, sql, params=()):
        """Executes a SELECT on remote database"""
        self.__raise_error_if_closed()

        self.row_count = 0
        self.__description = []

        the_cursor_util = CursorUtil()
        parms_dict = the_cursor_util.get_http_parameters_dict(params)

        is_prepared_statement = False
        if len(parms_dict) > 0:
            is_prepared_statement = True

        self.__result_set_info = self.__aceql_http_api.execute_query(sql, is_prepared_statement, parms_dict)

        # Appends the files to delete
        self.__filelist.append(self.__result_set_info.get_filename())

        self.__rowcount = self.__result_set_info.get_row_count()

        AceQLDebug.debug("self.rowcount: " + str(self.__rowcount))
        AceQLDebug.debug("filename    : " + self.__result_set_info.get_filename())

        # first build the description for Cursor.description
        self.__build_description()

        self.__row_parser = RowParser(self.__result_set_info.get_filename(), self.__result_set_info.get_row_count())

    def fetchone(self):
        """ Fetch the next row of a query result set, returning a single sequence,
        or None when no more data is available"""

        self.__raise_error_if_closed()
        row_available = self.__row_parser.build_next_row()

        if not row_available:
            return None

        values_per_column_index = self.__row_parser.get_values_per_col_index()
        types_per_column_index = self.__row_parser.get_types_per_col_index()

        if Parms.DEBUG_ON:
            print("values_per_column_index: " + str(values_per_column_index))
            print("types_per_column_index : " + str(types_per_column_index))

        the_list = []
        index = 0
        for k, v in values_per_column_index.items():
            # print(k, v)
            # detect Epoch timestamp and convert them to date for DATE and datetime for
            # TIMESTAMP
            if types_per_column_index[index] == "TIMESTAMP":
                the_list.append(DateTimeUtil.get_datetime_from_timestamp(v))
            elif types_per_column_index[index] == "DATE":
                the_list.append(DateTimeUtil.get_date_from_timestamp(v))
            elif types_per_column_index[index] == "TIME":
                the_list.append(DateTimeUtil.get_time_from_timestamp(v))
            else:
                the_list.append(v)

            index += 1

        the_tup = tuple(the_list)
        return the_tup

    def fetchmany(self, size=-1):
        """Fetch the next set of rows of a query result, returning a sequence of sequences

        (e.g. a list of tuples). An empty sequence is returned when no more rows are available.

        The number of rows to fetch per call is specified by the parameter.
        If it is not given, the cursor's arraysize determines the number of rows to be fetched."""

        self.__raise_error_if_closed()
        size_to_use = size
        if size_to_use <= 1:
            size_to_use = self.arraysize

        list_tuples = []
        cpt = 0
        while True:
            the_tup = self.fetchone()
            cpt += 1
            if the_tup is None:
                break
            list_tuples.append(the_tup)
            if cpt >= size_to_use:
                return list_tuples

        return list_tuples

    def fetchall(self):
        """Fetches all (remaining) rows of a query result, returning a list.

            Note that the cursors arraysize attribute can affect the performance
            of this operation. An empty list is returned when no rows are available.
        """
        self.__raise_error_if_closed()

        list_tuples = []
        while True:
            the_tup = self.fetchone()
            if the_tup is None:
                break
            list_tuples.append(the_tup)

        return list_tuples

    def setinputsizes(self, sizes):
        """ Does nothing. Implemented for respect to PEP 249."""
        pass

    def setoutputsize(self, sizes, column=None):
        """ Does nothing. Implemented for respect to PEP 249."""
        pass

    def __build_description(self):
        """ Builds the .description property"""

        self.__raise_error_if_closed()
        self.__description = []

        if self.__rowcount < 1:
            return

        row_parser = None

        try:
            row_parser = RowParser(self.__result_set_info.get_filename(), self.__rowcount)
            row_parser.build_next_row()  # read first row to get the column names, only way to do it...

            aceql_types = row_parser.get_types_per_col_index()
            aceql_names = row_parser.column_names_per_index()

            AceQLDebug.debug("aceql_types : " + str(aceql_types))
            AceQLDebug.debug("aceql_names: " + str(aceql_names))

            index = 0

            while index < len(aceql_types):
                name_and_type = list()

                #name_and_type.append(aceql_names[index])
                #name_and_type.append(aceql_types[index])

                aceql_name = aceql_names.get(index)
                if aceql_name is not None:
                    name_and_type.append(aceql_name)

                name_and_type.append(aceql_types.get(index))

                # Append 5 non set values
                name_and_type.append(None)
                name_and_type.append(None)
                name_and_type.append(None)
                name_and_type.append(None)
                name_and_type.append(None)

                the_tup = tuple(name_and_type)
                self.__description.append(the_tup)
                index += 1

        finally:
            if row_parser is not None:
                row_parser.close()

    # def blob_download(self, blob_id, filename, total_length=0,
    # progress_holder=None):
    # 	""" Allows to download a blob corresponding to a blob_id into a filename
    #
    # 		total_length & and a ProgressHolder instance may be passed for progress
    # 		indication.
    # 	"""
    #
    # 	if blob_id is None:
    # 		raise TypeError("blob_id is null!")
    #
    # 	if filename is None:
    # 		raise TypeError("filename is null!")
    #
    # 	self.__aceql_http_api.blob_download(blob_id, filename, total_length,
    # 	progress_holder)

    def get_blob_length(self, column_index):
        """ Gets the remote BLOB length  on a column in the current row

        To be used if progress indicator needed.

        """
        self.__raise_error_if_closed()

        if column_index is None:
            raise TypeError("column_index is null!")

        values_per_column_index = self.__row_parser.get_values_per_col_index()

        AceQLDebug.debug("values_per_column_index: " + str(values_per_column_index))

        if values_per_column_index is None:
            raise Error("Not positioned on a row. (No fetchone call done.)",
                        0, None, None, 200)

        blob_id = values_per_column_index[column_index]

        print("blob_id: " + str(blob_id))

        if blob_id is None:
            raise Error("No value found for column_index " + str(column_index),
                        0, None, None, 200)

        if not blob_id.endswith(".blob"):
            raise Error("Fetched value does not correspond to a BLOB Id: " + str(blob_id),
                        0, None, None, 200)

        blob_length = self.__aceql_http_api.get_blob_length(blob_id)
        return blob_length

    def get_blob_stream(self, column_index):
        """ Returns a BLOB stream on a column in the current row.

            The column index starts at 0.
        """

        self.__raise_error_if_closed()

        if column_index is None:
            raise TypeError("column_index is null!")

        values_per_column_index = self.__row_parser.get_values_per_col_index()

        if values_per_column_index is None:
            raise Error("Not positioned on a row. (Seems no fetchone() call done.)",
                        0, None, None, 200)

        blob_id = values_per_column_index[column_index]

        if blob_id is None:
            raise Error("No value found for column_index " + str(column_index),
                        0, None, None, 200)

        if not blob_id.endswith(".blob"):
            raise Error("Fetched value does not correspond to a BLOB Id: " + str(blob_id),
                        0, None, None, 200)

        # OK!  we have a valid BLOB Id:
        response = self.__aceql_http_api.get_blob_stream(blob_id)
        return response

    def close(self):
        """ Closes the cursor and releases underlying stream & file resource (result set). """

        if self.__is_closed:
            return

        self.__is_closed = True

        if self.__row_parser is not None:
            self.__row_parser.close()  # very important

        if Parms.DELETE_FILES:
            for filename in self.__filelist:
                os.remove(filename)

    def __raise_error_if_closed(self):
        if self.__is_closed:
            raise Error("Invalid call: Cursor is closed.", 0, None, None, 200)


#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from dataclasses import dataclass
import marshmallow_dataclass

from aceql.metadata.jdbc_database_meta_data import JdbcDatabaseMetaData


@dataclass
class JdbcDatabaseMetaDataDto(object):
    status: str
    jdbcDatabaseMetaData: JdbcDatabaseMetaData

    class Meta:
        ordered = True

    def __str__(self):
        """ The string representation."""
        return str(self.status) + ", " + str(self.jdbcDatabaseMetaData)




#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

import unittest

from aceql._private.datetime_util import *


class DateTimeUtilTest(unittest.TestCase):
    def test_A(self):

        the_datetime = datetime.now()
        print("theDatetime       : " + str(the_datetime))

        timestamp_str = DateTimeUtil.get_timestamp_from_date(the_datetime)
        print("timestamp_str     : " + timestamp_str)

        the_datetime_new = DateTimeUtil.get_datetime_from_timestamp(timestamp_str)
        print("the_datetime_new  : " + str(the_datetime_new))

        timestamp_str_new = DateTimeUtil.get_timestamp_from_date(the_datetime_new)
        print("timestamp_str_new : " + timestamp_str_new)

        self.assertEqual(timestamp_str, timestamp_str_new)

    def print_datetime_values(self):
        the_datetime = datetime.now()
        the_date = the_datetime.date()
        the_time = the_datetime.time()

        print()
        print("the_datetime: " + str(type(the_datetime)))
        print("the_date    : " + str(type(the_date)))
        print("the_time    : " + str(type(the_time)))
        print()
        print("the_datetime: " + str(the_datetime))
        print("the_date    : " + str(the_date))
        print("the_time    : " + str(the_time))
        print()
        print("the_time hour   : " + str(the_time.hour))
        print("the_time minute : " + str(the_time.minute))
        print("the_time second : " + str(the_time.second))
        print("the_time micros : " + str(the_time.microsecond))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##
from typing import List

import aceql
from aceql import *

import unittest
import sys

from aceql.metadata.column import Column
from aceql.metadata.remote_database_metadata import RemoteDatabaseMetaData
import webbrowser
import os


class TestAll(unittest.TestCase):
    def test_A(self):

        print(sys.version)
        # assert sys.version_info >= (2,5)
        print()

        print("aceql.apilevel    : " + aceql.apilevel)
        print("aceql.threadsafety: " + str(aceql.threadsafety))
        print("aceql.paramstyle  : " + aceql.paramstyle)

        proxies = None
        auth = None

        use_proxy = False
        if use_proxy:
            proxies = {
                "http": "http://localhost:8080",
            }

            auth = TestAll.getProxyAuth()

        localhost = "http://localhost:9090/aceql"
        server_host = "https://www.aceql.com:9443/aceql"
        server_host_no_ssl = "http://www.aceql.com:9090/aceql"

        host = localhost

        Connection.set_timeout(10)
        Connection.set_stateless(False)
        connection = aceql.connect(host, "sampledb", "user1", "password1", proxies=proxies, auth=auth)
        connection.set_gzip_result(True)

        print()
        print("aceql version: " + connection.get_client_version())
        print()

        remote_database_meta_data = RemoteDatabaseMetaData(connection)

        filename = os.path.expanduser("~") + os.sep + "db_schema.html"
        remote_database_meta_data.db_schema_download(filename)

        do_webbrowser = True
        if do_webbrowser is True:
            webbrowser.open('file://' + os.path.realpath(filename))
        print("Done db_schema_download!")

        jdbc_meta_data = remote_database_meta_data.get_jdbc_database_meta_data()
        print("Major Version: " + str(jdbc_meta_data.getJDBCMajorVersion))
        print("Minor Version: " + str(jdbc_meta_data.getJDBCMinorVersion))
        print("IsReadOnly   : " + str(jdbc_meta_data.isReadOnly))

        print(jdbc_meta_data.getURL)
        print(jdbc_meta_data)
        print("Done get_jdbc_database_meta_data!")

        print()

        print("Get the table names:");
        table_names = remote_database_meta_data.get_table_names()

        print("Print the column details of each table:")
        for table_name in table_names:
            table = remote_database_meta_data.get_table(table_name)

            print()
            print("Columns of table: " + table_name)
            for column in table.columns:
                print(column)

        connection.close()

if __name__ == '__main__':
    unittest.main()



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

import collections
import json
import os
from io import open


class ColumnTypesBuilder(object):
    """Allows to build the dictionary of SQL types per coumn index of a
       valid result set file (no check is done)"""

    def __init__(self, filename):
        self.__filename = filename

        if filename is None:
            raise TypeError("filename is null!")

        if not os.path.isfile(filename):
            raise IOError("filename does not exist: " + str(filename))

    def get_types_per_col_index(self):
        with open(self.__filename, 'r') as fd:
            s = ""
            while True:
                line = fd.readline()
                if line == '':
                    break
                line = line.strip()
                # end of "column_types" array: close the JSON and quit
                if line == "],":
                    s += "]}"
                    break
                s += line

            # AceQLDebug.print(s)
            j = json.loads(s, object_pairs_hook=collections.OrderedDict)
            list_column_types = j["column_types"]

            # print("list_column_types: " + str(list_column_types))

            dict_column_types = {}

            # index = 0
            # for s in list_column_types:
            # 	dict_column_types[index] = s
            # 	index +=1

            index = 0
            while index < len(list_column_types):
                dict_column_types[index] = list_column_types[index]
                index += 1

            # print("dict_column_types: " + str(dict_column_types))

            return dict_column_types



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##


class Parms(object):
    """Parameters for behavior."""
    DEBUG_ON = False
    DELETE_FILES = True
    PRINT_PROGRESS_INDICATOR = True



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

from aceql._private.parms import  *


class ProgressIndicator(object):
    """Class that hold a progress when transferring a Blob."""

    def __init__(self):
        self.__percent = 0
        self.__cancelled = False

    #
    # Private methods,, not to be called.
    #

    def _increment(self):
        """ Called by AceQL internal code during transfer progress."""
        if self.__percent < 99:
            self.__percent += 1
        if Parms.PRINT_PROGRESS_INDICATOR:
            print(str(self.__percent) + "%")

    def _set_complete(self):
        """ To be called by upload/download AceQL internal code when transfer is complete."""
        self.__percent = 100

    #
    # Public methods
    #
    @property
    def get_percent(self):
        """ Allows to get transfer progress from 0 to 100 """
        return self.__percent

    def cancel(self):
        """ Allows caller to cancel the transfer operation """
        self.__cancelled = True

    def is_cancelled(self):
        """ Allows for AceQL internal for transfer thread to test if cancelled """
        return self.__cancelled



# -*- coding: utf-8 -*-
#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2020,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.                                
#                                                                               
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
##

import unittest

from datetime import datetime, date, time
from aceql.sql_null_type import *
from aceql._private.cursor_util import *


class CursorUtilTest(unittest.TestCase):
    def test_A(self):
        the_datetime = datetime.now()
        the_date = date(2017, 10, 31)
        the_time = the_datetime.time()

        # for NULL values
        tup_null_integer = None, SqlNullType.INTEGER

        the_list = [tup_null_integer, 1, 12.53, True, "text", the_datetime, the_date, the_time]

        cpt = 0
        for x in the_list:
            print()
            print(str(x) + " / type: " + str(type(x)))
            sql_type = CursorUtil.get_sql_type(x)
            print("sql_type : " + sql_type)

            if cpt == 0:
                self.assertEqual(sql_type, "TYPE_NULL4")

            if cpt == 1:
                self.assertEqual(sql_type, "INTEGER")

            if cpt == 2:
                self.assertEqual(sql_type, "REAL")

            if cpt == 3:
                self.assertEqual(sql_type, "BIT")

            if cpt == 4:
                self.assertEqual(sql_type, "VARCHAR")

            if cpt == 5:
                self.assertEqual(sql_type, "TIMESTAMP")

            if cpt == 6:
                self.assertEqual(sql_type, "DATE")

            if cpt == 7:
                self.assertEqual(sql_type, "TIME")

            cpt += 1


if __name__ == '__main__':
    unittest.main()
